# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('conditions', '0004_initial'),
        ('devices', '0001_initial'),
        ('diagnosticreports', '0003_initial'),
        ('medicationadministrations', '0003_initial'),
        ('observations', '0002_initial'),
        ('patients', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('procedures', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='medicationadministrationpartof',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_part_of_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='medicationadministration',
            name='part_of',
            field=models.ManyToManyField(blank=True, related_name='medication_administration_part_of', to='medicationadministrations.medicationadministrationpartof'),
        ),
        migrations.AddField(
            model_name='medicationadministrationperformer',
            name='function',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_performer_function', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='medicationadministration',
            name='performer',
            field=models.ManyToManyField(blank=True, related_name='medication_administration_performer', to='medicationadministrations.medicationadministrationperformer'),
        ),
        migrations.AddField(
            model_name='medicationadministrationperformeractorcodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_performer_actor_codeable_reference_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='medicationadministrationperformer',
            name='actor',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_performer_actor', to='medicationadministrations.medicationadministrationperformeractorcodeablereference'),
        ),
        migrations.AddField(
            model_name='medicationadministrationperformeractorreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_performer_actor_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='medicationadministrationperformeractorreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_performer_actor_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationadministrationperformeractorreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_performer_actor_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='medicationadministrationperformeractorreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_performer_actor_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='medicationadministrationperformeractorreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_performer_actor_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='medicationadministrationperformeractorreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_performer_actor_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='medicationadministrationperformeractorcodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_performer_actor_codeable_reference_reference', to='medicationadministrations.medicationadministrationperformeractorreference'),
        ),
        migrations.AddField(
            model_name='medicationadministrationreasoncodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_reason_codeable_reference_codeable_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='medicationadministration',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='medication_administration_reason_codeable_reference', to='medicationadministrations.medicationadministrationreasoncodeablereference'),
        ),
        migrations.AddField(
            model_name='medicationadministrationreasonreference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_reason_reference_condition', to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='medicationadministrationreasonreference',
            name='diagnostic_report',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_reason_reference_diagnostic_report', to='diagnosticreports.diagnosticreport'),
        ),
        migrations.AddField(
            model_name='medicationadministrationreasonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_reason_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationadministrationreasonreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_reason_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='medicationadministrationreasonreference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_reason_reference_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='medicationadministrationreasoncodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_reason_codeable_reference_reference', to='medicationadministrations.medicationadministrationreasonreference'),
        ),
    ]



"""inventory item serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer
from rest_framework import serializers

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    QuantitySerializer,
    RangeSerializer,
    RatioSerializer,
)
from dfhir.base.serializers import QuantitySerializer as DurationSerializer
from dfhir.inventoryitems.models import (
    InventoryItem,
    InventoryItemAssociation,
    InventoryItemAssociationTypeReference,
    InventoryItemCharacteristic,
    InventoryItemDescription,
    InventoryItemInstance,
    InventoryItemName,
    InventoryItemProductReference,
    InventoryItemReference,
    InventoryItemResponsibleOrganization,
)
from dfhir.locations.serializers import LocationReferenceSerializer
from dfhir.patients.serializers import PatientOrganizationReferenceSerializer


class InventoryItemAssociationTypeReferenceSerializer(BaseReferenceModelSerializer):
    """inventory item association type reference serializer."""

    identifier = IdentifierSerializer(many=False, required=True)

    class Meta:
        """meta options."""

        model = InventoryItemAssociationTypeReference
        exclude = ["created_at", "updated_at"]


class InventoryItemName(serializers.ModelSerializer):
    """inventory item name serializer."""

    class Meta:
        """meta options."""

        model = InventoryItemName
        exclude = ["created_at", "updated_at"]


class InventoryItemResponsibleAssociationSerializer(WritableNestedModelSerializer):
    """inventory item responsible association serializer."""

    code = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = InventoryItemResponsibleOrganization
        exclude = ["created_at", "updated_at"]


class InventoryItemDescriptionSerializer(serializers.ModelSerializer):
    """inventory item description serializer."""

    class Meta:
        """meta options."""

        model = InventoryItemDescription
        exclude = ["created_at", "updated_at"]


class InventoryItemAssociationSerializer(WritableNestedModelSerializer):
    """inventory item association serializer."""

    association_type = CodeableConceptSerializer(many=False, required=False)
    related_item = InventoryItemAssociationTypeReferenceSerializer(
        many=False, required=False
    )
    quantity = RatioSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = InventoryItemAssociation
        exclude = ["created_at", "updated_at"]


class InventoryItemCharacteristicSerializer(WritableNestedModelSerializer):
    """inventory item characteristic serializer."""

    characteristic_type = CodeableConceptSerializer(many=False, required=False)
    value_quantity = QuantitySerializer(many=False, required=False)
    value_range = RangeSerializer(many=False, required=False)
    value_ratio = RatioSerializer(many=False, required=False)
    value_annotation = AnnotationSerializer(many=False, required=False)
    value_duration = DurationSerializer(many=False, required=False)
    value_codeable_concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = InventoryItemCharacteristic
        exclude = ["created_at", "updated_at"]


class InventoryItemInstanceSerializer(WritableNestedModelSerializer):
    """inventory item instance serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    subject = PatientOrganizationReferenceSerializer(many=False, required=False)
    location = LocationReferenceSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = InventoryItemInstance
        exclude = ["created_at", "updated_at"]


class InventoryItemProductReference(BaseReferenceModelSerializer):
    """inventory item product reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = InventoryItemProductReference
        exclude = ["created_at", "updated_at"]


class InventoryItemSerializer(BaseWritableNestedModelSerializer):
    """inventory item serializer."""

    identifier = IdentifierSerializer(many=False, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    code = CodeableConceptSerializer(many=True, required=False)
    name = InventoryItemName(many=True, required=False)
    responsible_organization = InventoryItemResponsibleAssociationSerializer(
        many=True, required=False
    )
    description = InventoryItemDescriptionSerializer(many=False, required=False)
    inventory_status = CodeableConceptSerializer(many=False, required=False)
    base_unit = CodeableConceptSerializer(many=False, required=False)
    net_content = QuantitySerializer(many=False, required=False)
    association = InventoryItemAssociationSerializer(many=True, required=False)
    characteristic = InventoryItemCharacteristicSerializer(many=True, required=False)
    instance = InventoryItemInstanceSerializer(many=False, required=False)
    product_reference = InventoryItemProductReference(many=False, required=False)

    class Meta:
        """meta options."""

        model = InventoryItem
        exclude = ["created_at", "updated_at"]


class InventoryItemReferenceSerializer(BaseReferenceModelSerializer):
    """inventory item reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = InventoryItemReference
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('groups', '0001_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PersonalRelationshipSourceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PersonalRelationship',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('asserter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personal_relationships_asserter', to='base.organizationreference')),
                ('confidence', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personal_relationships_confidence', to='base.codeableconcept')),
                ('group', models.ManyToManyField(blank=True, related_name='personal_relationships_group', to='groups.groupreference')),
                ('period', models.ManyToManyField(blank=True, related_name='personal_relationships_period', to='base.period')),
                ('relationship_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='personal_relationships_relationship_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PersonalRelationshipAsserterReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personal_relationship_asserter_reference_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personal_relationship_asserter_reference_organization', to='organizations.organization')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personal_relationship_asserter_reference_patient', to='patients.patient')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('activitydefinitions', '0001_initial'),
        ('base', '0001_initial'),
        ('locations', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='activitydefinition',
            name='as_needed_codeable_concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_as_needed_codeable_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='author',
            field=models.ManyToManyField(blank=True, related_name='activity_definition_author', to='base.contactdetail'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='body_site',
            field=models.ManyToManyField(blank=True, related_name='activity_definition_body_site', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='contact',
            field=models.ManyToManyField(blank=True, related_name='activity_definition_contact', to='base.contactdetail'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='editor',
            field=models.ManyToManyField(blank=True, related_name='activity_definition_editor', to='base.contactdetail'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='effective_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='endorser',
            field=models.ManyToManyField(blank=True, related_name='activity_definition_endorser', to='base.contactdetail'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='activity_definition_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='jurisdiction',
            field=models.ManyToManyField(blank=True, related_name='activity_definition_jurisdiction', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='kind',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_kind', to='base.coding'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='location',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_location', to='locations.locationreference'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='product_codeable_concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_product_codeable_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='quantity',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_quantity', to='base.simplequantity'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='related_artifact',
            field=models.ManyToManyField(blank=True, related_name='activity_definition_related_artifact', to='base.relatedartifact'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='reviewer',
            field=models.ManyToManyField(blank=True, related_name='activity_definition_reviewer', to='base.contactdetail'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='subject_codealbe_concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_subject_codeable_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='timing_age',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_timing_age', to='base.age'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='timing_duration',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_timing_duration', to='base.duration'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='timing_range',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_timing_range', to='base.range'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='timing_timing',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_timing_timing', to='base.timing'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='topic',
            field=models.ManyToManyField(blank=True, related_name='activity_definition_topic', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='use_context',
            field=models.ManyToManyField(blank=True, related_name='activity_definition_use_context', to='base.usagecontext'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='version_algorithm_coding',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_version_algorithm_coding', to='base.coding'),
        ),
        migrations.AddField(
            model_name='activitydefinitiondynamicvalue',
            name='expression',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_dynamic_value_expression', to='base.expression'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='dynamic_value',
            field=models.ManyToManyField(blank=True, related_name='activity_definition_note', to='activitydefinitions.activitydefinitiondynamicvalue'),
        ),
        migrations.AddField(
            model_name='activitydefinitionparticipant',
            name='function',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_participant_function', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='activitydefinitionparticipant',
            name='role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_participant_role', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='activitydefinition',
            name='participant',
            field=models.ManyToManyField(blank=True, related_name='activity_definition_participant', to='activitydefinitions.activitydefinitionparticipant'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('deviceassociations', '0003_initial'),
        ('devices', '0001_initial'),
        ('groups', '0001_initial'),
        ('patients', '0001_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='deviceassociationoperationoperatorreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='device_association_operation_operator_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='deviceassociationoperationoperatorreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='device_association_operation_operator_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='deviceassociationoperation',
            name='operator',
            field=models.ManyToManyField(blank=True, related_name='device_association_operation_operator', to='deviceassociations.deviceassociationoperationoperatorreference'),
        ),
        migrations.AddField(
            model_name='deviceassociationsubjectreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_association_subject_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='deviceassociationsubjectreference',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_association_subject_reference_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='deviceassociationsubjectreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_association_subject_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='deviceassociationsubjectreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_association_subject_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='deviceassociationsubjectreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_association_subject_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='deviceassociationsubjectreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_association_subject_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='deviceassociation',
            name='subject',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='device_association_subject', to='deviceassociations.deviceassociationsubjectreference'),
        ),
    ]



"""Plandefinitions urls."""

from django.urls import path

from . import views

app_name = "plandefinitions"


urlpatterns = [
    path("plandefinitions/", views.PlanDefinitionListView.as_view()),
    path("plandefinitions/<int:pk>/", views.PlanDefinitionDetailView.as_view()),
]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('careteams', '0002_initial'),
        ('organizations', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='GoalAddressesReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GoalSourceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GoalSubjectReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GoalTarget',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('detail_string', models.CharField(max_length=255, null=True)),
                ('detail_boolean', models.BooleanField(default=False)),
                ('detail_integer', models.IntegerField(null=True)),
                ('due_date', models.DateField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GoalAcceptance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('agree', 'Agree'), ('disagree', 'Disagree'), ('pending', 'Pending')], max_length=255)),
                ('priority', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='goal_acceptance_priority', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Goal',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('lifecycle_status', models.CharField(choices=[('proposed', 'Proposed'), ('planned', 'Planned'), ('accepted', 'Accepted'), ('active', 'Active'), ('on-hold', 'On Hold'), ('completed', 'Completed'), ('cancelled', 'Cancelled'), ('entered-in-error', 'Entered in Error'), ('rejected', 'Rejected')], max_length=255)),
                ('continuous', models.BooleanField(default=False)),
                ('start_date', models.DateField(null=True)),
                ('status_date', models.DateField(null=True)),
                ('achievement_status', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='goal_achievement_status', to='base.codeableconcept')),
                ('category', models.ManyToManyField(blank=True, related_name='goal_category', to='base.codeableconcept')),
                ('description', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='goal_description', to='base.codeableconcept')),
                ('identifier', models.ManyToManyField(blank=True, related_name='goal_identifier', to='base.identifier')),
                ('note', models.ManyToManyField(blank=True, related_name='goal_note', to='base.annotation')),
                ('priority', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='goal_priority', to='base.codeableconcept')),
                ('start_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='goal_start_code', to='base.codeableconcept')),
                ('status_reason', models.ManyToManyField(blank=True, related_name='goal_status_code', to='base.codeableconcept')),
                ('acceptance', models.ManyToManyField(blank=True, related_name='goal_acceptance', to='goals.goalacceptance')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GoalAcceptanceParticipantReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('care_team', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='goal_acceptance_participant_reference_care_team', to='careteams.careteam')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='goal_acceptance_participant_reference_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='goal_acceptance_participant_reference_organization', to='organizations.organization')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Encounter serializers file."""

from drf_writable_nested.serializers import WritableNestedModelSerializer
from rest_framework import serializers

from dfhir.appointments.serializers import AppointmentReferenceSerializer
from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
    VirtualServiceDetailsSerializer,
)
from dfhir.base.serializers import (
    QuantitySerializer as DurationSerializer,
)
from dfhir.careteams.serializers import CareTeamReferenceSerializer
from dfhir.conditions.serializers import ConditionCodeableReferenceSerializer
from dfhir.healthcareservices.serializers import (
    HealthCareServiceCodeableReferenceSerializer,
)
from dfhir.locations.serializers import (
    LocationOrganizationReferenceSerializer,
    LocationReferenceSerializer,
)
from dfhir.patients.serializers import PatientGroupReferenceSerializer

from .models import (
    DietPreference,
    Encounter,
    EncounterAdmission,
    EncounterBasedOnReference,
    EncounterCondition,
    EncounterDiagnosis,
    EncounterEpisodeOfCareReference,
    EncounterLocation,
    EncounterParticipant,
    EncounterParticipantActorReference,
    EncounterReason,
    EncounterReasonValueCodeableReference,
    EncounterReasonValueReference,
    EncounterReference,
    SpecialArrangement,
    SpecialCourtesy,
)


class EncounterParticipantActorReferenceSerializer(BaseReferenceModelSerializer):
    """Encounter Participant Actor Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = EncounterParticipantActorReference
        exclude = ["created_at", "updated_at"]


class EncounterParticipantSerializer(WritableNestedModelSerializer):
    """Encounter Participant Serializer."""

    actor = EncounterParticipantActorReferenceSerializer(many=False, required=False)
    type = CodeableConceptSerializer(many=True, required=False)
    period = PeriodSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = EncounterParticipant
        exclude = ["created_at", "updated_at"]


class EncounterDiagnosisSerializer(WritableNestedModelSerializer):
    """Encounter Diagnosis Serializer."""

    use = CodeableConceptSerializer(many=True, required=False)
    condition = ConditionCodeableReferenceSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = EncounterDiagnosis
        exclude = ["created_at", "updated_at"]


class EncounterAdmissionSerializer(WritableNestedModelSerializer):
    """Encounter Admission Serializer."""

    pre_admission_identifier = IdentifierSerializer(many=False, required=False)
    origin = LocationOrganizationReferenceSerializer(many=False, required=False)
    admit_source = CodeableConceptSerializer(many=False, required=False)
    re_admission = CodeableConceptSerializer(many=False, required=False)
    destination = LocationOrganizationReferenceSerializer(many=False, required=False)
    discharge_disposition = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = EncounterAdmission
        exclude = ["created_at", "updated_at"]


class EncounterLocationSerializer(WritableNestedModelSerializer):
    """Encounter Location Serializer."""

    location = LocationReferenceSerializer(many=False, required=False)
    form = CodeableConceptSerializer(many=False, required=False)
    period = PeriodSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = EncounterLocation
        exclude = ["created_at", "updated_at"]


class EncounterBasedOnReferenceSerializer(BaseReferenceModelSerializer):
    """Encounter Based On Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = EncounterBasedOnReference
        exclude = ["created_at", "updated_at"]


class EncounterReferenceSerializer(BaseReferenceModelSerializer):
    """Encounter Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = EncounterReference
        exclude = ["created_at", "updated_at"]


class EncounterReasonValueReferenceSerializer(BaseReferenceModelSerializer):
    """Encounter Reason Value Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = EncounterReasonValueReference
        exclude = ["created_at", "updated_at"]


class EncounterReasonValueCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Encounter Reason Value Codeable Reference Serializer."""

    reference = EncounterReasonValueReferenceSerializer(many=False, required=False)
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = EncounterReasonValueCodeableReference
        exclude = ["created_at", "updated_at"]


class EncounterReasonSerializer(WritableNestedModelSerializer):
    """Encounter Reason Serializer."""

    use = CodeableConceptSerializer(many=True, required=False)
    value = EncounterReasonValueCodeableReferenceSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = EncounterReason
        exclude = ["created_at", "updated_at"]


class EncounterConditionSerializer(serializers.ModelSerializer):
    """Encounter Condition Serializer."""

    class Meta:
        """Meta class."""

        model = EncounterCondition
        exclude = ["created_at", "updated_at"]


class SpecialCourtesySerializer(serializers.ModelSerializer):
    """Encounter Specialty Serializer."""

    class Meta:
        """Meta class."""

        model = SpecialCourtesy
        exclude = ["created_at", "updated_at"]


class DietPreferenceSerializer(serializers.ModelSerializer):
    """Diet Preference Serializer."""

    class Meta:
        """Meta class."""

        model = DietPreference
        exclude = ["created_at", "updated_at"]


class SpecialArrangementSerializer(serializers.ModelSerializer):
    """Special Arrangement Serializer."""

    class Meta:
        """Meta class."""

        model = SpecialArrangement
        exclude = ["created_at", "updated_at"]


class EncounterSerializer(BaseWritableNestedModelSerializer):
    """Encounter Serializer."""

    def get_fields(self):
        """Get fields."""
        from dfhir.accounts.serializers import AccountReferenceSerializer
        from dfhir.episodeofcares.serializers import EpisodeOfCareReferenceSerializer

        fields = super().get_fields()

        fields["account"] = AccountReferenceSerializer(required=False, many=True)
        fields["episode_of_care"] = EpisodeOfCareReferenceSerializer(
            required=False, many=True
        )
        return fields

    identifier = IdentifierSerializer(required=False, many=True)
    klass = CodeableConceptSerializer(required=False, many=True)
    priority = CodeableConceptSerializer(required=False)
    type = CodeableConceptSerializer(required=False, many=True)
    service_type = HealthCareServiceCodeableReferenceSerializer(
        required=False, many=True
    )
    subject = PatientGroupReferenceSerializer(required=False)
    subject_status = CodeableConceptSerializer(required=False)
    based_on = EncounterBasedOnReferenceSerializer(required=False, many=True)
    care_team = CareTeamReferenceSerializer(required=False, many=True)
    part_of = EncounterReferenceSerializer(required=False, many=True)
    service_provider = OrganizationReferenceSerializer(required=False)
    participant = EncounterParticipantSerializer(required=False, many=True)
    appointment = AppointmentReferenceSerializer(required=False, many=True)
    virtual_service = VirtualServiceDetailsSerializer(required=False, many=True)
    actual_period = PeriodSerializer(required=False)
    length = DurationSerializer(required=False)
    reason = EncounterReasonSerializer(required=False, many=True)
    diagnosis = EncounterDiagnosisSerializer(required=False, many=True)
    diet_preference = CodeableConceptSerializer(required=False, many=True)
    special_arrangement = CodeableConceptSerializer(required=False, many=True)
    special_courtesy = CodeableConceptSerializer(required=False, many=True)
    admission = EncounterAdmissionSerializer(required=False)
    location = EncounterLocationSerializer(required=False, many=True)

    class Meta:
        """Meta class."""

        model = Encounter
        exclude = ["created_at", "updated_at"]
        rename_fields = {
            "class": "klass",
        }


class EncounterEpisodeOfCareReferenceSerializer(BaseReferenceModelSerializer):
    """Encounter Episode Of Care Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = EncounterEpisodeOfCareReference
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-05-02 14:24

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('medicationknowledges', '0001_initial'),
        ('medications', '0001_initial'),
        ('substances', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='medication',
            name='definition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_definition', to='medicationknowledges.medicationknowledgereference'),
        ),
        migrations.AddField(
            model_name='medicationsubstancereference',
            name='substance',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_substance_reference_substance', to='substances.substance'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('careplans', '0004_initial'),
        ('encounters', '0003_initial'),
        ('nutritionorders', '0002_initial'),
        ('patients', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('servicerequests', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='VisionPrescriptionLensSpecificationPrism',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('base', models.CharField(choices=[('up', 'Up'), ('down', 'Down'), ('in', 'In'), ('out', 'Out')], default='up', max_length=255)),
                ('amount', models.FloatField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='VisionPrescriptionBasedOnReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('care_plan', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='vision_prescription_based_on_reference_care_plan', to='careplans.careplan')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='vision_prescription_based_on_reference_identifier', to='base.identifier')),
                ('nutrition_order', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='vision_prescription_based_on_reference_nutrition_order', to='nutritionorders.nutritionorder')),
                ('service_request', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='vision_prescription_based_on_reference_service_request', to='servicerequests.servicerequest')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='VisionPrescriptionLensSpecification',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('eye', models.CharField(choices=[('right', 'Right'), ('left', 'Left')], default='right', max_length=255)),
                ('sphere', models.FloatField(null=True)),
                ('cylinder', models.FloatField(null=True)),
                ('axis', models.FloatField(null=True)),
                ('add', models.FloatField(null=True)),
                ('power', models.FloatField(null=True)),
                ('back_curve', models.FloatField(null=True)),
                ('diameter', models.FloatField(null=True)),
                ('color', models.CharField(max_length=255, null=True)),
                ('brand', models.CharField(max_length=255, null=True)),
                ('duration', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='vision_prescription_lens_specification_duration', to='base.simplequantity')),
                ('note', models.ManyToManyField(blank=True, related_name='vision_prescription_lens_specification_note', to='base.annotation')),
                ('product', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='vision_prescription_lens_specification_product', to='base.codeableconcept')),
                ('prism', models.ManyToManyField(blank=True, related_name='vision_prescription_lens_specification_prism', to='visionprescriptions.visionprescriptionlensspecificationprism')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='VisionPrescription',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('cancelled', 'Cancelled'), ('draft', 'Draft'), ('entered-in-error', 'Entered in Error')], default='active', max_length=255)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('date_written', models.DateTimeField(null=True)),
                ('encounter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='vision_prescription_encounter', to='encounters.encounterreference')),
                ('identifier', models.ManyToManyField(blank=True, related_name='vision_prescription_identifier', to='base.identifier')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='vision_prescription_patient', to='patients.patientreference')),
                ('prescriber', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='vision_prescription_prescriber', to='practitioners.practitionerpractitionerrolereference')),
                ('based_on', models.ManyToManyField(blank=True, related_name='vision_prescription_based_on', to='visionprescriptions.visionprescriptionbasedonreference')),
                ('lens_specification', models.ManyToManyField(blank=True, related_name='vision_prescription_lens_specification', to='visionprescriptions.visionprescriptionlensspecification')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Careplan choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class CareplanStatusChoices(models.TextChoices):
    """Careplan status choices."""

    DRAFT = "draft", _("Draft")
    ACTIVE = "active", _("Active")
    COMPLETED = "completed", _("Completed")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")
    UNKNOWN = "unknown", _("Unknown")
    REVOKED = "revoked", _("Revoked")
    ON_HOLD = "on-hold", _("On Hold")


class CarePlanIntentChoices(models.TextChoices):
    """Careplan intent choices."""

    PROPOSAL = "proposal", _("Proposal")
    PLAN = "plan", _("Plan")
    ORDER = "order", _("Order")
    OPTION = "option", _("Option")
    DIRECTIVE = "directive", _("Directive")



"""Devicedispenses serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    ReferenceSerializer,
    SimpleQuantitySerializer,
)

# from dfhir.devices.serializers import DeviceDeviceDefinitionCodeableReferenceSerializer
from dfhir.careplans.serializers import CarePlanDeviceRequestReferenceSerializer
from dfhir.detectedissues.serializers import DetectedIssueCodeableReferenceSerializer
from dfhir.devices.serializers import DeviceDeviceDefinitionCodeableReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.locations.serializers import LocationReferenceSerializer
from dfhir.patients.serializers import PatientPractitionerReferenceSerializer
from dfhir.procedures.serializers import ProcedureReferenceSerializer
from dfhir.provenances.serializers import ProvenanceReferenceSerializer

from .models import (
    DeviceDispense,
    DeviceDispensePerformer,
    DeviceDispensePerformerActorReference,
    DeviceDispenseReceiverReference,
)

# from dfhir.detectedissues.serializers import DetectedIssueCodeableReferenceSerializer


class DeviceDispenseReceiverReferenceSerializer(BaseReferenceModelSerializer):
    """DeviceDispenseReceiverReference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = DeviceDispenseReceiverReference
        exclude = ["created_at", "updated_at"]


class DeviceDispensePerformerActorReferenceSerializer(BaseReferenceModelSerializer):
    """DeviceDispensePerformerActorReference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = DeviceDispensePerformerActorReference
        exclude = ["created_at", "updated_at"]


class DeviceDispensePerformerSerializer(WritableNestedModelSerializer):
    """DeviceDispensePerformer serializer."""

    function = CodeableConceptSerializer(many=False, required=False)
    actor = DeviceDispensePerformerActorReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = DeviceDispensePerformer
        exclude = ["created_at", "updated_at"]


class DeviceDispenseSerializer(BaseWritableNestedModelSerializer):
    """DeviceDispense serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    based_on = CarePlanDeviceRequestReferenceSerializer(many=True, required=False)
    part_of = ProcedureReferenceSerializer(many=True, required=False)
    status_reason = DetectedIssueCodeableReferenceSerializer(many=False, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    device = DeviceDeviceDefinitionCodeableReferenceSerializer(
        many=False, required=False
    )
    subject = PatientPractitionerReferenceSerializer(many=False, required=False)
    receiver = DeviceDispenseReceiverReferenceSerializer(many=False, required=False)
    encounter = EncounterReferenceSerializer(many=False, required=False)
    supporting_information = ReferenceSerializer(many=True, required=False)
    performer = DeviceDispensePerformerSerializer(many=True, required=False)
    location = LocationReferenceSerializer(many=False, required=False)
    type = CodeableConceptSerializer(many=False, required=False)
    quantity = SimpleQuantitySerializer(many=False, required=False)
    destination = LocationReferenceSerializer(many=False, required=False)
    note = AnnotationSerializer(many=True, required=False)
    event_history = ProvenanceReferenceSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = DeviceDispense
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-05-05 14:45

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coverages', '0002_initial'),
        ('devicerequests', '0004_initial'),
        ('supplyrequests', '0001_initial'),
        ('tasks', '0001_initial'),
        ('transports', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='transport',
            name='insurance',
            field=models.ManyToManyField(blank=True, related_name='transport_insurance', to='coverages.coverageclaimresponsereference'),
        ),
        migrations.AddField(
            model_name='transportbasedonreference',
            name='device_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='transport_based_on_device_request', to='devicerequests.devicerequest'),
        ),
        migrations.AddField(
            model_name='transportbasedonreference',
            name='display',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='transportbasedonreference',
            name='reference',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='transportbasedonreference',
            name='supply_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='transport_based_on_supply_request', to='supplyrequests.supplyrequest'),
        ),
        migrations.AddField(
            model_name='transportbasedonreference',
            name='task',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='transport_based_on_task', to='tasks.task'),
        ),
        migrations.AddField(
            model_name='transportbasedonreference',
            name='type',
            field=models.CharField(max_length=255, null=True),
        ),
    ]



# Generated by Django 5.2 on 2025-05-02 14:24

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('chargeitems', '0005_rename_charitemreference_chargeitemreference'),
        ('invoices', '0003_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoicelineitem',
            name='charge_item_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_line_item_charge_item_reference', to='chargeitems.chargeitemreference'),
        ),
    ]



"""diagnostic reports views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.generics import ListAPIView
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.diagnosticreports.models import (
    ConclusionCode,
    DiagnosticCategory,
    DiagnosticReport,
    DiagnosticReportCode,
)
from dfhir.diagnosticreports.serializers import (
    ConclusionCodeSerializer,
    DiagnosticCategorySerializer,
    DiagnosticReportCodeSerializer,
    DiagnosticReportSerializer,
)


class DiagnosticReportListView(APIView):
    """diagnostic report list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: DiagnosticReportSerializer(many=True)})
    def get(self, request):
        """Get all diagnostic reports."""
        diagnostic_reports = DiagnosticReport.objects.all()
        serializer = DiagnosticReportSerializer(diagnostic_reports, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=DiagnosticReportSerializer, responses={201: DiagnosticReportSerializer}
    )
    def post(self, request):
        """Create a diagnostic report."""
        serializer = DiagnosticReportSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class DiagnosticReportDetailView(APIView):
    """diagnostic report detail view."""

    def get_object(self, pk):
        """Get a diagnostic report object."""
        try:
            return DiagnosticReport.objects.get(pk=pk)
        except DiagnosticReport.DoesNotExist as error:
            raise Http404 from error

    @extend_schema(responses={200: DiagnosticReportSerializer})
    def get(self, request, pk=None):
        """Get a diagnostic report."""
        diagnostic_report = self.get_object(pk)
        serializer = DiagnosticReportSerializer(diagnostic_report)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200, DiagnosticReportSerializer})
    def patch(self, request, pk=None):
        """Update a diagnostic report."""
        diagnostic_report = self.get_object(pk)
        serializer = DiagnosticReportSerializer(
            diagnostic_report, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete(self, request, pk=None):
        """Delete a diagnostic report."""
        diagnostic_report = self.get_object(pk)
        diagnostic_report.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)


@extend_schema(responses={200: DiagnosticReportCodeSerializer})
class DiagnosticReportCodeListView(ListAPIView):
    """diagnostic report code list view."""

    serializer_class = DiagnosticReportCodeSerializer
    permission_classes = [AllowAny]
    queryset = DiagnosticReportCode.objects.all()


@extend_schema(responses={200: DiagnosticCategorySerializer})
class DiagnosticCategoryListView(ListAPIView):
    """diagnostic category list view."""

    serializer_class = DiagnosticCategorySerializer
    permission_classes = [AllowAny]
    queryset = DiagnosticCategory.objects.all()


@extend_schema(responses={200: ConclusionCodeSerializer})
class ConclusionCodeListView(ListAPIView):
    """conclusion code list view."""

    serializer_class = ConclusionCodeSerializer
    permission_classes = [AllowAny]
    queryset = ConclusionCode.objects.all()



# Generated by Django 5.2 on 2025-04-23 12:46

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('careteams', '0001_initial'),
        ('conditions', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='careteam',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='care_teams_reason_code', to='conditions.conditioncodeablereference'),
        ),
    ]



"""Devicerequests app views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import DeviceRequest
from .serializers import DeviceRequestSerializer


class DeviceRequestListView(APIView):
    """DeviceRequest list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: DeviceRequestSerializer(many=True)})
    def get(self, request):
        """Get a list of device requests."""
        device_requests = DeviceRequest.objects.all()
        serializer = DeviceRequestSerializer(device_requests, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=DeviceRequestSerializer, responses={201: DeviceRequestSerializer}
    )
    def post(self, request):
        """Create a device request."""
        serializer = DeviceRequestSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class DeviceRequestDetailView(APIView):
    """Device request detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get a care plan object."""
        try:
            return DeviceRequest.objects.get(pk=pk)
        except DeviceRequest.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: DeviceRequestSerializer})
    def get(self, request, pk=None):
        """Get a care plan."""
        device_request = self.get_object(pk)
        serializer = DeviceRequestSerializer(device_request)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=DeviceRequestSerializer, responses={200: DeviceRequestSerializer}
    )
    def patch(self, request, pk=None):
        """Update a care plan."""
        device_request = self.get_object(pk)
        serializer = DeviceRequestSerializer(
            device_request, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete a care plan."""
        device_request = self.get_object(pk)
        device_request.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('careteams', '0004_initial'),
        ('devices', '0001_initial'),
        ('locations', '0001_initial'),
        ('medicationdispenses', '0002_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('procedures', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='medicationdispense',
            name='part_of',
            field=models.ManyToManyField(blank=True, related_name='medication_dispense_part_of', to='procedures.procedurereference'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='quantity',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_dispense_quantity', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_dispense_subject', to='patients.patientgroupreference'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='supporting_information',
            field=models.ManyToManyField(blank=True, related_name='medication_dispense_supporting_information', to='base.reference'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_dispense_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='medicationdispenseperformer',
            name='function',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_dispense_performer_function', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='performer',
            field=models.ManyToManyField(blank=True, related_name='medication_dispense_performer', to='medicationdispenses.medicationdispenseperformer'),
        ),
        migrations.AddField(
            model_name='medicationdispenseperformeractorreference',
            name='care_team',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='careteams.careteam'),
        ),
        migrations.AddField(
            model_name='medicationdispenseperformeractorreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='devices.device'),
        ),
        migrations.AddField(
            model_name='medicationdispenseperformeractorreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationdispenseperformeractorreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='medicationdispenseperformeractorreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='patients.patient'),
        ),
        migrations.AddField(
            model_name='medicationdispenseperformeractorreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='medicationdispenseperformeractorreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='medicationdispenseperformeractorreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='medicationdispenseperformer',
            name='actor',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_dispense_performer_actor', to='medicationdispenses.medicationdispenseperformeractorreference'),
        ),
        migrations.AddField(
            model_name='medicationdispensereceiverreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationdispensereceiverreference',
            name='location',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='locations.location'),
        ),
        migrations.AddField(
            model_name='medicationdispensereceiverreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='patients.patient'),
        ),
        migrations.AddField(
            model_name='medicationdispensereceiverreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='medicationdispensereceiverreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='medicationdispensereceiverreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='receiver',
            field=models.ManyToManyField(blank=True, related_name='medication_dispense_receiver', to='medicationdispenses.medicationdispensereceiverreference'),
        ),
        migrations.AddField(
            model_name='medicationdispensesubstitution',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='medication_dispense_substitution_reason', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='medicationdispensesubstitution',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_dispense_substitution_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='substitution',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_dispense_substitution', to='medicationdispenses.medicationdispensesubstitution'),
        ),
        migrations.AddField(
            model_name='medicationdispensesubstitutionresponsiblepartyreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationdispensesubstitutionresponsiblepartyreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='medicationdispensesubstitutionresponsiblepartyreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='medicationdispensesubstitutionresponsiblepartyreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='medicationdispensesubstitution',
            name='responsible_party',
            field=models.ManyToManyField(blank=True, related_name='medication_dispense_substitution_responsible_party', to='medicationdispenses.medicationdispensesubstitutionresponsiblepartyreference'),
        ),
    ]



"""Enrollment Requests Views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import EnrollmentRequest
from .serializers import EnrollmentRequestSerializer


class EnrollmentRequestListView(APIView):
    """Enrollment Request list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: EnrollmentRequestSerializer(many=True)})
    def get(self, request):
        """Get all enrollment requests."""
        enrollment_requests = EnrollmentRequest.objects.all()
        serializer = EnrollmentRequestSerializer(enrollment_requests, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=EnrollmentRequestSerializer,
        responses={201: EnrollmentRequestSerializer},
    )
    def post(self, request):
        """Create an enrollment request."""
        serializer = EnrollmentRequestSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class EnrollmentRequestDetailView(APIView):
    """Enrollment Request detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get enrollment request object."""
        try:
            return EnrollmentRequest.objects.get(id=pk)
        except EnrollmentRequest.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: EnrollmentRequestSerializer})
    def get(self, request, pk=None):
        """Get enrollment request detail."""
        enrollment_request = self.get_object(pk)
        serializer = EnrollmentRequestSerializer(enrollment_request)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=EnrollmentRequestSerializer,
        responses={200: EnrollmentRequestSerializer},
    )
    def patch(self, request, pk=None):
        """Update enrollment request."""
        enrollment_request = self.get_object(pk)
        serializer = EnrollmentRequestSerializer(
            enrollment_request, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete enrollment request."""
        enrollment_request = self.get_object(pk)
        enrollment_request.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""supply requests models."""

from django.db import models

from dfhir.base.models import BaseReference, Identifier, TimeStampedModel
from dfhir.supplyrequests.choices import (
    SupplyRequestIntentChoices,
    SupplyRequestPriorityChoices,
    SupplyRequestStatusChoices,
)


class SupplyRequestParameter(TimeStampedModel):
    """supply request parameter model."""

    code = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        related_name="supply_request_parameter_code",
        null=True,
    )
    value_codeable_concept = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        related_name="supply_request_parameter_value_codeable_concept",
    )
    value_quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.DO_NOTHING,
        related_name="supply_request_parameter_value_quantity",
        null=True,
    )
    value_range = models.ForeignKey(
        "base.Range",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_parameter_value_range",
    )
    value_boolean = models.BooleanField(null=True)


class SupplyRequestRequesterReference(BaseReference):
    """supply request requester reference model."""

    identifier = models.ForeignKey(
        "base.Identifier",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_requester_reference_identifier",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_requester_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_reques_requestert_reference_practitioner_role",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_requester_reference_organization",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_requester_reference_patient",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_requester_reference_related_person",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_requester_reference_device",
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_requester_reference_care_team",
    )


class SupplyRequestSupplierReference(BaseReference):
    """supply request supplier reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_supplier_reference_identifier",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_supplier_reference_organization",
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthcareService",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_supplier_reference_healthcare_service",
    )


class SupplyRequestReasonReference(BaseReference):
    """supply request reason reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_reason_reference_identifier",
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_reason_reference_condition",
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_reason_reference_observation",
    )
    diagnostic_report = models.ForeignKey(
        "diagnosticreports.DiagnosticReport",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_reason_reference_diagnostic_report",
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_reason_reference_document_reference",
    )


class SupplyRequestReasonCodeableReference(TimeStampedModel):
    """supply request reason codeable reference model."""

    concept = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_reason_codeable_reference_concept",
    )
    reference = models.ForeignKey(
        SupplyRequestReasonReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_reason_codeable_reference_reference",
    )


class SupplyRequestDeliverFromReference(BaseReference):
    """supply request deliver from reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_deliver_from_reference_identifier",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_deliver_from_reference_organization",
    )
    location = models.ForeignKey(
        "locations.Location",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_deliver_from_reference_location",
    )


class SupplyRequestDeliverToReference(BaseReference):
    """supply request deliver to reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_deliver_to_reference_identifier",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_deliver_to_reference_organization",
    )
    location = models.ForeignKey(
        "locations.Location",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_deliver_to_reference_location",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_deliver_to_reference_patient",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_deliver_to_reference_related_person",
    )


class SupplyRequestItemReference(BaseReference):
    """supply request reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_item_reference_identifier",
    )
    medication = models.ForeignKey(
        "medications.Medication",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_item_reference_medication",
    )
    substance = models.ForeignKey(
        "substances.Substance",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_item_reference_substance",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_item_reference_device",
    )
    device_definition = models.ForeignKey(
        "devicedefinitions.DeviceDefinition",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_item_reference_device_definition",
    )
    biologically_derived_product = models.ForeignKey(
        "biologicallyderivedproducts.BiologicallyDerivedProduct",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_item_reference_biologically_derived_product",
    )
    nutrition_product = models.ForeignKey(
        "nutritionproducts.NutritionProduct",
        on_delete=models.DO_NOTHING,
        related_name="supply_request_item_reference_nutrition_order",
        null=True,
    )
    inventory_item = models.ForeignKey(
        "inventoryitems.InventoryItem",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_item_reference_inventory_item",
    )


class SupplyRequestItemCodeableReference(TimeStampedModel):
    """supply request item codeable reference model."""

    concept = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        related_name="supply_request_item_codeable_reference_concept",
        null=True,
    )
    reference = models.ForeignKey(
        SupplyRequestItemReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_item_codeable_reference_reference",
    )


class SupplyRequest(TimeStampedModel):
    """supply request model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="supply_request_identifier"
    )
    status = models.CharField(
        max_length=200, null=True, choices=SupplyRequestStatusChoices.choices
    )
    intent = models.CharField(
        max_length=255, null=True, choices=SupplyRequestIntentChoices.choices
    )
    based_on = models.ManyToManyField(
        "base.Reference", related_name="supply_request_based_on", blank=True
    )
    category = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_category",
    )
    priority = models.CharField(
        max_length=255, null=True, choices=SupplyRequestPriorityChoices.choices
    )
    deliver_for = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_deliver_for",
    )
    item = models.ForeignKey(
        SupplyRequestItemCodeableReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_item",
    )
    quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_quantity",
    )
    parameter = models.ManyToManyField(
        SupplyRequestParameter, blank=True, related_name="supply_request_parameter"
    )
    occurrence_date_time = models.DateTimeField(null=True, blank=True)
    occurrence_period = models.ForeignKey(
        "base.Period",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_occurrence_period",
    )
    occurrence_timing = models.ForeignKey(
        "base.Timing",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_occurrence_timing",
    )
    authored_on = models.DateTimeField(null=True, blank=True)
    requester = models.ForeignKey(
        SupplyRequestRequesterReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_requester",
    )
    supplier = models.ManyToManyField(
        SupplyRequestSupplierReference,
        blank=True,
        related_name="supply_request_supplier",
    )
    reason = models.ManyToManyField(
        SupplyRequestReasonCodeableReference,
        blank=True,
        related_name="supply_request_reason",
    )
    deliver_from = models.ForeignKey(
        SupplyRequestDeliverFromReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_deliver_from",
    )
    deliver_to = models.ForeignKey(
        SupplyRequestDeliverToReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_deliver_to",
    )


class SupplyRequestReference(BaseReference):
    """supply request reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=False,
        related_name="supply_request_reference_identifier",
    )
    supply_request = models.ForeignKey(
        SupplyRequest,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_request_reference_supply_request",
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('accounts', '0003_initial'),
        ('base', '0005_initial'),
        ('encounters', '0003_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0002_initial'),
        ('paymentreconciliations', '0001_initial'),
        ('practitionerroles', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='paymentreconciliationallocation',
            name='responsible',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='payment_reconciliation_allocation_responsible', to='practitionerroles.practitionerrolereference'),
        ),
        migrations.AddField(
            model_name='paymentreconciliationallocation',
            name='submitter',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_allocation_submitter', to='paymentreconciliations.organizationpractitionerpractitionerrolereference'),
        ),
        migrations.AddField(
            model_name='paymentreconciliationallocation',
            name='target_item_identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_allocation_target_item_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='paymentreconciliationallocation',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_allocation_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='paymentreconciliation',
            name='allocation',
            field=models.ManyToManyField(blank=True, related_name='payment_reconciliation_allocation', to='paymentreconciliations.paymentreconciliationallocation'),
        ),
        migrations.AddField(
            model_name='paymentreconciliationallocationtargetreference',
            name='account',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_allocation_target_reference_account', to='accounts.account'),
        ),
        migrations.AddField(
            model_name='paymentreconciliationallocationtargetreference',
            name='encounter',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_allocation_target_reference_encounter', to='encounters.encounter'),
        ),
        migrations.AddField(
            model_name='paymentreconciliationallocationtargetreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='payment_reconciliation_allocation_target_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='paymentreconciliationallocation',
            name='target',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_allocation_target', to='paymentreconciliations.paymentreconciliationallocationtargetreference'),
        ),
        migrations.AddField(
            model_name='paymentreconciliationpaymentissuerreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='payment_reconciliation_payment_issuer_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='paymentreconciliationpaymentissuerreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_payment_issuer_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='paymentreconciliationpaymentissuerreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_payment_issuer_reference_patient', to='patients.patientreference'),
        ),
        migrations.AddField(
            model_name='paymentreconciliationpaymentissuerreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_payment_issuer_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='paymentreconciliation',
            name='payment_issuer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_payment_issuer', to='paymentreconciliations.paymentreconciliationpaymentissuerreference'),
        ),
        migrations.AddField(
            model_name='paymentreconciliation',
            name='process_note',
            field=models.ManyToManyField(blank=True, related_name='payment_reconciliation_process_note', to='paymentreconciliations.paymentreconciliationprocessnote'),
        ),
        migrations.AddField(
            model_name='paymentreconciliationreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='paymentreconciliationreference',
            name='payment_reconciliation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_reference_payment_reconciliation', to='paymentreconciliations.paymentreconciliation'),
        ),
    ]



"""Patient views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import Patient
from .serializers import PatientSerializer


class PatientListView(APIView):
    """Patient list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: PatientSerializer(many=True)})
    def get(self, request, pk=None):
        """Get all patients."""
        queryset = Patient.objects.all()
        serializer = PatientSerializer(queryset, many=True)
        return Response(serializer.data)

    @extend_schema(request=PatientSerializer, responses={200: PatientSerializer})
    def post(self, request):
        """Create a patient."""
        request_data = request.data
        request_data.pop("user", None)

        serializer = PatientSerializer(data=request_data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)


class PatientDetailView(APIView):
    """Patient detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get patient object."""
        try:
            return Patient.objects.get(pk=pk)
        except Patient.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: PatientSerializer})
    def get(self, request, pk=None):
        """Get a patient."""
        queryset = self.get_object(pk)
        serializer = PatientSerializer(queryset)
        return Response(serializer.data)

    @extend_schema(responses={200: PatientSerializer})
    def patch(self, request, pk=None):
        """Update a patient."""
        queryset = self.get_object(pk)
        serializer = PatientSerializer(queryset, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    def delete(self, request, pk=None):
        """Delete a patient."""
        patient = self.get_object(pk)
        patient.user.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



# Generated by Django 5.2 on 2025-04-23 12:46

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('inactive', 'Inactive'), ('entered-in-error', 'Entered in Error'), ('on-hold', 'On Hold'), ('unknown', 'Unknown')], max_length=255, null=True)),
                ('name', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
                ('calculated_at', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AccountBalance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('estimated', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AccountCoverage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('priority', models.PositiveIntegerField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AccountDiagnosis',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.PositiveIntegerField(null=True)),
                ('date_of_diagnosis', models.DateTimeField(null=True)),
                ('on_admission', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AccountGuarantor',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('on_hold', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AccountProcedure',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.PositiveIntegerField(null=True)),
                ('date_of_service', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AccountReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AccountRelatedAccount',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AccountsGuarantorPartyReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AccountSubjectReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""practitioner role views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.practitionerroles.models import PractitionerRole, PractitionerRoleCode
from dfhir.practitionerroles.serializers import (
    PractitionerRoleCodeSerializer,
    PractitionerRoleSerializer,
    PractitionerRoleWithPractitionerIdSerializer,
)


class PractitionerRoleListView(APIView):
    """Practitioner role list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: PractitionerRoleWithPractitionerIdSerializer})
    def get(self, request, pk=None):
        """Get practitioner roles."""
        queryset = PractitionerRole.objects.all()
        serializer = PractitionerRoleSerializer(queryset, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=PractitionerRoleSerializer,
        responses={200: PractitionerRoleWithPractitionerIdSerializer},
    )
    def post(self, request):
        """Create a practitioner role."""
        request_data = request.data
        serializer = PractitionerRoleSerializer(data=request_data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)


class PractitionerRoleDetailView(APIView):
    """Practitioner role detail view."""

    def get_object(self, pk):
        """Get practitioner role object."""
        try:
            return PractitionerRole.objects.get(pk=pk)
        except PractitionerRole.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: PractitionerRoleWithPractitionerIdSerializer})
    def get(self, request, pk=None):
        """Get a practitioner role."""
        queryset = self.get_object(pk)
        serializer = PractitionerRoleSerializer(queryset)
        return Response(serializer.data)

    @extend_schema(responses={200: PractitionerRoleWithPractitionerIdSerializer})
    def patch(self, request, pk=None):
        """Update a practitioner role."""
        queryset = self.get_object(pk)
        serializer = PractitionerRoleSerializer(
            queryset, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    def delete(self, request, pk=None):
        """Delete a practitioner role."""
        practitioner_role = self.get_object(pk)
        practitioner_role.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)


class PractitionerRoleCodeListView(APIView):
    """Practitioner role code list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: PractitionerRoleCodeSerializer})
    def get(self, request, pk=None):
        """Get practitioner role codes."""
        queryset = PractitionerRoleCode.objects.all()
        serializer = PractitionerRoleCodeSerializer(queryset, many=True)
        return Response(serializer.data)



"""observation models."""

from django.db import models

from dfhir.base.models import (
    Annotation,
    Attachment,
    BaseReference,
    CodeableConcept,
    Identifier,
    Period,
    Quantity,
    Range,
    Ratio,
    Reference,
    TimeStampedModel,
    Timing,
)
from dfhir.bodystructures.models import BodyStructureReference
from dfhir.devices.models import DeviceDeviceMetricReference
from dfhir.encounters.models import EncounterReference
from dfhir.molecularsequences.models import MolecularSequenceReference
from dfhir.observationdefinitions.models import ObservationDefinitionReference
from dfhir.patients.models import Patient, PatientGroupReference
from dfhir.practitioners.models import Practitioner

from .choices import (
    ObservationStatus,
    TriggeredByType,
)


class ObservationReference(BaseReference):
    """observation reference model."""

    identifier = models.ForeignKey(Identifier, on_delete=models.SET_NULL, null=True)
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_reference_observation",
    )


class ObservationBodyStructureReference(BaseReference):
    """observation body structure reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_body_structure_reference_identifier",
    )
    # body_structure = models.ForeignKey(
    #     "BodyStructure",
    #     on_delete=models.SET_NULL,
    #     null=True,
    #     related_name="observation_body_structure_reference_body_structure",
    # )


class ObservationBasedOnReference(BaseReference):
    """observation based on reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_based_on_reference_identifier",
    )
    # TODO: care_plan = models.ForeignKey("CarePlan", on_delete=models.SET_NULL, null=True)
    # TODO: device_request = models.ForeignKey("DeviceRequest", on_delete=models.SET_NULL, null=True)
    # TODO: immunization_recommendation = models.ForeignKey("ImmunizationRecommendation", on_delete=models.SET_NULL, null=True)
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_based_on_reference_medication_request",
    )
    nutrition_order = models.ForeignKey(
        "nutritionorders.NutritionOrder",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_based_on_reference_nutrition_order",
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_based_on_reference_service_request",
    )


class ObservationPartOfReference(BaseReference):
    """observation part of reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_part_of_reference_identifier",
    )
    medication_administration = models.ForeignKey(
        "medicationadministrations.MedicationAdministration",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_part_of_reference_medication_administration",
    )
    medication_dispense = models.ForeignKey(
        "medicationdispenses.MedicationDispense",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_part_of_reference_medication_dispense",
    )
    medication_statement = models.ForeignKey(
        "medicationstatements.MedicationStatement",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_part_of_reference_medication_statement",
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_part_of_reference_procedure",
    )
    immunization = models.ForeignKey(
        "immunizations.Immunization",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_part_of_reference_immunization",
    )
    imaging_study = models.ForeignKey(
        "imagingstudies.ImagingStudy",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_part_of_reference_imaging_study",
    )
    genomic_study = models.ForeignKey(
        "genomicstudies.GenomicStudy",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_part_of_reference_genomic_study",
    )


class ObservationSubjectReference(BaseReference):
    """reference to the subject of the observation."""

    identifier = models.ForeignKey(Identifier, on_delete=models.SET_NULL, null=True)
    patient = models.ForeignKey(Patient, on_delete=models.SET_NULL, null=True)
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_subject_reference_group",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_subject_reference_device",
    )
    location = models.ForeignKey(
        "locations.Location", on_delete=models.SET_NULL, null=True
    )
    organization = models.ForeignKey(
        "organizations.Organization", on_delete=models.SET_NULL, null=True
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_subject_reference_procedure",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner", on_delete=models.SET_NULL, null=True
    )
    medication = models.ForeignKey(
        "medications.Medication", on_delete=models.SET_NULL, null=True
    )
    # TODO: substance = models.ForeignKey(
    #     "substances.Substance", on_delete=models.SET_NULL, null=True
    # )
    biologically_derived_product = models.ForeignKey(
        "biologicallyderivedproducts.BiologicallyDerivedProduct",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_subject_reference_biologically_derived_product",
    )
    nutrition_product = models.ForeignKey(
        "nutritionproducts.NutritionProduct",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_subject_reference_nutrition_product",
    )


class ObservationTriggeredBy(TimeStampedModel):
    """triggered by model."""

    observation = models.ForeignKey(
        ObservationReference,
        on_delete=models.CASCADE,
        related_name="triggered_by_observation",
    )
    type = models.CharField(max_length=255, null=True, choices=TriggeredByType.choices)
    reason = models.TextField(null=True)


class ObservationHasMemberReference(BaseReference):
    """observation has member reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_has_member_reference_identifier",
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_has_member_reference_observation",
    )
    # TODO: questionnaire_response = models.ForeignKey("QuestionnaireResponse", on_delete=models.SET_NULL, null=True)
    molecular_sequence = models.ForeignKey(
        "molecularsequences.MolecularSequence",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_has_member_reference_molecular_sequence",
    )


class ObservationDerivedFromReference(BaseReference):
    """observation derived from reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_derived_from_reference_identifier",
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_derived_from_reference_document_reference",
    )
    imaging_study = models.ForeignKey(
        "imagingstudies.ImagingStudy", on_delete=models.SET_NULL, null=True
    )
    imaging_selection = models.ForeignKey(
        "imagingselections.ImagingSelection", on_delete=models.SET_NULL, null=True
    )
    # TODO: questionnaire_response = models.ForeignKey("QuestionnaireResponse", on_delete=models.SET_NULL, null=True)
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_derived_from_reference_observation",
    )
    molecular_sequence = models.ForeignKey(
        "molecularsequences.MolecularSequence",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_derived_from_reference_molecular_sequence",
    )
    # geometric_study= models.ForeignKey("geometricstudies.GeometricStudy", on_delete=models.SET_NULL, null=True)


class SampledDataIntervalUnitCodes(TimeStampedModel):
    """sampled data interval unit codes model."""

    name = models.CharField(max_length=255, null=True)
    definition = models.TextField(null=True)
    source = models.CharField(max_length=255, null=True)


class SampledData(TimeStampedModel):
    """sampled data model."""

    origin = models.ForeignKey(
        Quantity,
        on_delete=models.SET_NULL,
        null=True,
        related_name="sampled_data_origin",
    )
    interval = models.DecimalField(max_digits=10, null=True, decimal_places=2)
    interval_unit = models.ForeignKey(
        SampledDataIntervalUnitCodes,
        on_delete=models.SET_NULL,
        null=True,
        related_name="sampled_data_interval_unit",
    )

    factor = models.DecimalField(max_digits=10, null=True, decimal_places=2)
    lower_limit = models.DecimalField(max_digits=10, null=True, decimal_places=2)
    upper_limit = models.DecimalField(max_digits=10, null=True, decimal_places=2)
    dimensions = models.IntegerField()
    # TODO: code_map = models.ForeignKey("CodeMap", null=True, on_delete=models.SET_NULL)
    offsets = models.CharField(max_length=255, null=True)
    data = models.TextField(null=True)


class ObservationPerformerReference(BaseReference):
    """observation performer reference model."""

    identifier = models.ForeignKey(Identifier, on_delete=models.SET_NULL, null=True)
    practitioner = models.ForeignKey(
        Practitioner,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_performer_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_performer_reference_practitioner_role",
    )
    organization = models.ForeignKey(
        "organizations.Organization", on_delete=models.SET_NULL, null=True
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_performer_reference_care_team",
    )
    patient = models.ForeignKey(Patient, on_delete=models.SET_NULL, null=True)
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.SET_NULL,
        null=True,
    )


class ObservationReferenceRange(TimeStampedModel):
    """reference range model."""

    low = models.ForeignKey(
        Quantity,
        on_delete=models.SET_NULL,
        null=True,
        related_name="reference_range_low",
    )
    high = models.ForeignKey(
        Quantity,
        null=True,
        on_delete=models.SET_NULL,
        related_name="reference_range_high",
    )
    normal_value = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="reference_range_normal_value",
    )

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="reference_range_type",
    )
    applies_to = models.ManyToManyField(
        CodeableConcept,
        related_name="reference_range_applies_to",
        blank=True,
    )
    age = models.ForeignKey(
        Range, on_delete=models.SET_NULL, null=True, related_name="reference_range_age"
    )
    text = models.TextField(null=True)


class ObservationComponent(TimeStampedModel):
    """observation component model."""

    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        null=True,
        related_name="observation_component_code",
    )
    value_quantity = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="observation_component_value",
    )
    value_codeable_concept = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.SET_NULL,
        related_name="observation_component_value_codeable_concept",
    )
    value_string = models.CharField(max_length=255, null=True)
    value_boolean = models.BooleanField(null=True)
    value_integer = models.IntegerField(null=True)
    value_range = models.ForeignKey(
        Range,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_component_value_range",
    )
    value_ratio = models.ForeignKey(
        Ratio,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_component_value_ratio",
    )
    value_sampled_data = models.ForeignKey(
        SampledData,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_component_value_sampled_data",
    )
    value_time = models.TimeField(null=True)
    value_date_time = models.DateTimeField(null=True)
    value_period = models.ForeignKey(
        Period,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_component_value_period",
    )
    value_attachment = models.ForeignKey(
        Attachment,
        null=True,
        on_delete=models.SET_NULL,
        related_name="observation_component_value_attachment",
    )
    value_reference = models.ForeignKey(
        MolecularSequenceReference, null=True, on_delete=models.SET_NULL
    )
    data_absent_reason = models.ForeignKey(
        CodeableConcept,
        related_name="observation_component_data_absent_reason",
        null=True,
        on_delete=models.SET_NULL,
    )
    interpretation = models.ManyToManyField(
        CodeableConcept,
        blank=True,
        related_name="observation_component_interpretation",
    )
    reference_range = models.ManyToManyField(ObservationReferenceRange, blank=True)


class Observation(TimeStampedModel):
    """observation model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="observation_identifier"
    )
    # TODO: instantiates_canonical = models.CharField(max_length=255, null=True)
    instantiates_reference = models.ForeignKey(
        ObservationDefinitionReference,
        null=True,
        on_delete=models.SET_NULL,
        related_name="observation_instantiates_reference",
    )
    based_on = models.ManyToManyField(
        ObservationBasedOnReference, related_name="observation_based_on", blank=True
    )
    triggered_by = models.ManyToManyField(
        ObservationTriggeredBy, blank=True, related_name="observation_triggered_by"
    )
    part_of = models.ManyToManyField(ObservationPartOfReference, blank=True)

    status = models.CharField(
        max_length=255, null=True, choices=ObservationStatus.choices
    )
    category = models.ManyToManyField(
        CodeableConcept, related_name="observation_category", blank=True
    )

    code = models.ForeignKey(CodeableConcept, on_delete=models.DO_NOTHING, null=True)
    subject = models.ForeignKey(
        ObservationSubjectReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="observation_subject",
    )
    focus = models.ManyToManyField(Reference, related_name="observation_focus")
    encounter = models.ForeignKey(
        EncounterReference, on_delete=models.DO_NOTHING, null=True
    )
    effective_date_time = models.DateTimeField(null=True)
    effective_period = models.ForeignKey(
        Period,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_effective_period",
    )
    effective_timing = models.ForeignKey(
        Timing,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_effective_timing",
    )
    effective_instant = models.DateTimeField(null=True)
    issued = models.DateTimeField(null=True)
    performer = models.ManyToManyField(
        ObservationPerformerReference, blank=True, related_name="observation_performer"
    )
    value_quantity = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="observation_value",
    )
    value_codeable_concept = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.SET_NULL,
        related_name="observation_value_codeable_concept",
    )
    value_string = models.CharField(max_length=255, null=True)
    value_boolean = models.BooleanField(null=True)
    value_integer = models.IntegerField(null=True)
    value_range = models.ForeignKey(
        Range,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_value_range",
    )
    value_ratio = models.ForeignKey(
        Ratio,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_value_ratio",
    )
    value_sampled_data = models.ForeignKey(
        SampledData,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_value_sampled_data",
    )
    value_time = models.TimeField(null=True)
    value_date_time = models.DateTimeField(null=True)
    value_period = models.ForeignKey(
        Period,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_value_period",
    )
    value_attachment = models.ForeignKey(
        Attachment,
        null=True,
        on_delete=models.SET_NULL,
        related_name="observation_value_attachment",
    )
    value_reference = models.ForeignKey(
        MolecularSequenceReference, null=True, on_delete=models.SET_NULL
    )
    data_absent_reason = models.ForeignKey(
        CodeableConcept,
        related_name="observation_data_absent_reason",
        null=True,
        on_delete=models.SET_NULL,
    )
    interpretation = models.ManyToManyField(
        CodeableConcept,
        blank=True,
        related_name="observation_interpretation",
    )
    note = models.ManyToManyField(
        Annotation, related_name="observation_note", blank=True
    )
    body_site = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_body_site",
    )
    body_structure = models.ForeignKey(
        BodyStructureReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_body_structure",
    )
    method = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_method",
    )
    specimen = models.ForeignKey(
        PatientGroupReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_specimen",
    )
    device = models.ForeignKey(
        DeviceDeviceMetricReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_device",
    )
    reference_range = models.ManyToManyField(
        ObservationReferenceRange,
        blank=True,
        related_name="observation_reference_range",
    )
    has_member = models.ManyToManyField(
        ObservationHasMemberReference, blank=True, related_name="observation_has_member"
    )
    derived_from = models.ManyToManyField(
        ObservationDerivedFromReference,
        blank=True,
        related_name="observation_derived_from",
    )
    component = models.ManyToManyField(
        ObservationComponent, blank=True, related_name="observation_component"
    )


class ObservationCodeableReference(TimeStampedModel):
    """observation codeable reference model."""

    reference = models.ForeignKey(
        ObservationReference,
        on_delete=models.CASCADE,
        related_name="observation_codeable_reference_reference",
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="observation_codeable_reference_concept",
        null=True,
    )


class DocumentReferenceObservationReference(BaseReference):
    """document reference observation reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="document_reference_observation_reference_identifier",
    )
    observation = models.ForeignKey(
        Observation,
        on_delete=models.SET_NULL,
        null=True,
        related_name="document_reference_observation_reference_observation",
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="document_reference_observation_reference_document_reference",
    )



"""Procedure views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import Procedure
from .serializers import ProcedureSerializer


class ProcedureListView(APIView):
    """Procedure List Views."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: ProcedureSerializer(many=True)})
    def get(self, request):
        """Get all procedures."""
        procedures = Procedure.objects.all()
        serializer = ProcedureSerializer(procedures, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=ProcedureSerializer, responses={201: ProcedureSerializer})
    def post(self, request):
        """Create a procedure."""
        serializer = ProcedureSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class ProcedureDetailView(APIView):
    """Procedure Detail View."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get a procedure object."""
        try:
            return Procedure.objects.get(pk=pk)
        except Procedure.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: ProcedureSerializer})
    def get(self, request, pk):
        """Get a procedure."""
        procedure = self.get_object(pk)
        serializer = ProcedureSerializer(procedure)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=ProcedureSerializer, responses={200: ProcedureSerializer})
    def patch(self, request, pk):
        """Update a procedure."""
        procedure = self.get_object(pk)
        serializer = ProcedureSerializer(procedure, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete a procedure."""
        procedure = self.get_object(pk)
        procedure.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Vision prescription models."""

from django.db import models

from dfhir.base.models import (
    Annotation,
    BaseReference,
    CodeableConcept,
    Identifier,
    SimpleQuantity,
    TimeStampedModel,
)

from . import choices


class VisionPrescriptionBasedOnReference(BaseReference):
    """Vision Prescription Based On Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="vision_prescription_based_on_reference_identifier",
        null=True,
    )
    care_plan = models.ForeignKey(
        "careplans.CarePlan",
        on_delete=models.DO_NOTHING,
        related_name="vision_prescription_based_on_reference_care_plan",
        null=True,
    )
    # request_orchestration = models.ForeignKey(
    #     "requestorchestrations.RequestOrchestration",
    #     on_delete=models.DO_NOTHING,
    #     related_name="vision_prescription_based_on_reference_request_orchestration",
    #     null=True,
    # )
    nutrition_order = models.ForeignKey(
        "nutritionorders.NutritionOrder",
        on_delete=models.DO_NOTHING,
        related_name="vision_prescription_based_on_reference_nutrition_order",
        null=True,
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.DO_NOTHING,
        related_name="vision_prescription_based_on_reference_service_request",
        null=True,
    )


class VisionPrescriptionLensSpecificationPrism(TimeStampedModel):
    """Vision Prescription Lens Specification Prism model."""

    base = models.CharField(
        max_length=255,
        choices=choices.VisionPrescriptionLensSpecificationPrismBaseChoices.choices,
        default=choices.VisionPrescriptionLensSpecificationPrismBaseChoices.UP,
    )
    amount = models.FloatField(null=True)


class VisionPrescriptionLensSpecification(TimeStampedModel):
    """Vision Prescription Lens Specification model."""

    product = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="vision_prescription_lens_specification_product",
        null=True,
    )
    eye = models.CharField(
        max_length=255,
        choices=choices.VisionPrescriptionLensSpecificationEyeChoices.choices,
        default=choices.VisionPrescriptionLensSpecificationEyeChoices.RIGHT,
    )
    sphere = models.FloatField(null=True)
    cylinder = models.FloatField(null=True)
    axis = models.FloatField(null=True)
    prism = models.ManyToManyField(
        VisionPrescriptionLensSpecificationPrism,
        related_name="vision_prescription_lens_specification_prism",
        blank=True,
    )
    add = models.FloatField(null=True)
    power = models.FloatField(null=True)
    back_curve = models.FloatField(null=True)
    diameter = models.FloatField(null=True)
    duration = models.ForeignKey(
        SimpleQuantity,
        on_delete=models.DO_NOTHING,
        related_name="vision_prescription_lens_specification_duration",
        null=True,
    )
    color = models.CharField(max_length=255, null=True)
    brand = models.CharField(max_length=255, null=True)
    note = models.ManyToManyField(
        Annotation,
        related_name="vision_prescription_lens_specification_note",
        blank=True,
    )


class VisionPrescription(TimeStampedModel):
    """Vision prescription model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="vision_prescription_identifier",
        blank=True,
    )
    based_on = models.ManyToManyField(
        VisionPrescriptionBasedOnReference,
        related_name="vision_prescription_based_on",
        blank=True,
    )
    status = models.CharField(
        max_length=255,
        choices=choices.VisionPrescriptionStatusChoices.choices,
        default=choices.VisionPrescriptionStatusChoices.ACTIVE,
    )
    created = models.DateTimeField(auto_now_add=True)
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.DO_NOTHING,
        related_name="vision_prescription_patient",
        null=True,
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.DO_NOTHING,
        related_name="vision_prescription_encounter",
        null=True,
    )
    date_written = models.DateTimeField(null=True)
    prescriber = models.ForeignKey(
        "practitioners.PractitionerPractitionerRoleReference",
        on_delete=models.DO_NOTHING,
        related_name="vision_prescription_prescriber",
        null=True,
    )
    lens_specification = models.ManyToManyField(
        VisionPrescriptionLensSpecification,
        related_name="vision_prescription_lens_specification",
        blank=True,
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('detectedissues', '0001_initial'),
        ('devices', '0001_initial'),
        ('encounters', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='detectedissue',
            name='encounter',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_encounter', to='encounters.encounterreference'),
        ),
        migrations.AddField(
            model_name='detectedissue',
            name='identified_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='base.period'),
        ),
        migrations.AddField(
            model_name='detectedissue',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='detected_issue_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='detectedissue',
            name='implicated',
            field=models.ManyToManyField(blank=True, related_name='detected_issue_implicated', to='base.reference'),
        ),
        migrations.AddField(
            model_name='detectedissueauthorreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_author_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='detectedissueauthorreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_author_reference_identifier', to='base.identifier'),
        ),
    ]



"""Choices for the appointments app."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class AppointmentType(models.TextChoices):
    """Appointment types."""

    ROUTINE = "routine", _("Routine")
    WALK_IN = "walk-in", _("Walk-in")
    CHECKUP = "checkup", _("Check-up")
    FOLLOWUP = "followup", _("Follow-up")
    EMERGENCY = "emergency", _("Emergency")


class AppointmentStatus(models.TextChoices):
    """Appointment statuses."""

    PROPOSED = "proposed", _("Proposed")
    PENDING = "pending", _("Pending")
    BOOKED = "booked", _("Booked")
    ARRIVED = "arrived", _("Arrived")
    FULFILLED = "fulfilled", _("Fulfilled")
    CANCELLED = "cancelled", _("Cancelled")
    NOSHOW = "noshow", _("No Show")
    ENTERED_IN_ERROR = "entered_in_error", _("Entered in Error")
    CHECKED_OUT = "checked_out", _("Checked Out")
    WAITLIST = "waitlist", _("Waitlist")


class AppointmentPriority(models.TextChoices):
    """Appointment priorities."""

    ASAP = "asap", _("ASAP")
    CALLBACK_RESULTS = "callback_results", _("Callback Results")
    ELECTIVE = "elective", _("Elective")
    EMERGENCY = "emergency", _("Emergency")
    PREOP = "preop", _("Preop")
    AS_NEEDED = "as_needed", _("As Needed")
    ROUTINE = "routine", _("Routine")
    RUSH_REPORTING = "rush_reporting", _("Rush Reporting")
    STAT = "stat", _("Stat")
    TIMING_CRITICAL = "timing_critical", _("Timing Critical")
    USE_AS_DIRECTED = "use_as_directed", _("Use As Directed")
    URGENT = "urgent", _("Urgent")
    CALLBACK_FOR_SCHEDULING = "callback_for_scheduling", _("Callback for Scheduling")
    CALLBACK_PLACER_FOR_SCHEDULING = (
        "callback_placer_for_scheduling",
        _("Callback Placer for Scheduling"),
    )
    CONTACT_PATIENT_FOR_SCHEDULING = (
        "contact_patient_for_scheduling",
        _("Contact Patient for Scheduling"),
    )


class RecurrenceType(models.TextChoices):
    """Recurrence types."""

    DAILY = "daily", _("Daily")
    WEEKLY = "weekly", _("Weekly")
    MONTHLY = "monthly", _("Monthly")
    YEARLY = "yearly", _("Yearly")


class ParticipationStatusChoices(models.TextChoices):
    """Participation statuses."""

    ACCEPTED = "accepted", _("Accepted")
    DECLINED = "declined", _("Declined")
    TENTATIVE = "tentative", _("Tentative")
    NEEDSACTION = "needs-action", _("Needs Action")


class WeekOfMonthChoices(models.TextChoices):
    """Week of month choices."""

    FIRST = "first", _("First")
    SECOND = "second", _("Second")
    THIRD = "third", _("Third")
    FOURTH = "fourth", _("Fourth")
    LAST = "last", _("Last")



"""Encounter views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.generics import ListAPIView
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .filters import EncounterFilter
from .models import (
    DietPreference,
    Encounter,
    EncounterCondition,
    SpecialArrangement,
    SpecialCourtesy,
)
from .serializers import (
    DietPreferenceSerializer,
    EncounterConditionSerializer,
    EncounterSerializer,
    SpecialArrangementSerializer,
    SpecialCourtesySerializer,
)


class EncounterListView(APIView):
    """Encounter List Views."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200, EncounterSerializer(many=True)})
    def get(self, request):
        """Get all encounters."""
        encounter = Encounter.objects.all()
        encounter_filters = EncounterFilter(request.GET, queryset=encounter)
        serializer = EncounterSerializer(encounter_filters.qs, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=EncounterSerializer, responses={201, EncounterSerializer})
    def post(self, request):
        """Create and encounter."""
        serializer = EncounterSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        encounter = serializer.save()
        detailed_serializer = EncounterSerializer(encounter)
        return Response(detailed_serializer.data, status=status.HTTP_201_CREATED)


class EncounterDetailView(APIView):
    """Encounter Serializer Detail View."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get encounter object."""
        try:
            return Encounter.objects.get(pk=pk)
        except Encounter.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200, EncounterSerializer})
    def get(self, request, pk=None):
        """Get an encounter."""
        encounter = self.get_object(pk)
        serializer = EncounterSerializer(encounter)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200, EncounterSerializer})
    def patch(self, request, pk=None):
        """Update and encounter."""
        encounter = self.get_object(pk=pk)
        serializer = EncounterSerializer(encounter, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete(self, request, pk=None):
        """Delete and encounter."""
        encounter = self.get_object(pk)
        encounter.delete()

        return Response(status=status.HTTP_204_NO_CONTENT)


@extend_schema(responses={200, EncounterConditionSerializer(many=True)})
class EncounterConditionListView(ListAPIView):
    """Encounter Condition List View."""

    model = EncounterCondition
    queryset = EncounterCondition.objects.all()
    permission_classes = [AllowAny]


@extend_schema(responses={200, DietPreferenceSerializer(many=True)})
class DietPreferenceListView(ListAPIView):
    """Diet Preference List View."""

    queryset = DietPreference.objects.all()
    permission_classes = [AllowAny]
    serializer_class = DietPreferenceSerializer


@extend_schema(responses={200, SpecialCourtesySerializer(many=True)})
class SpecialCourtesyListView(ListAPIView):
    """Special Preference List View."""

    queryset = SpecialCourtesy.objects.all()
    permission_classes = [AllowAny]
    serializer_class = SpecialCourtesySerializer


@extend_schema(responses={200, SpecialArrangementSerializer(many=True)})
class SpecialArrangementListView(ListAPIView):
    """Encounter Condition List View."""

    queryset = SpecialArrangement.objects.all()
    permission_classes = [AllowAny]
    serializer_class = SpecialArrangementSerializer



"""medication dispenses views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.medicationdispenses.models import MedicationDispense
from dfhir.medicationdispenses.serializers import MedicationDispenseSerializer


class MedicationDispenseListView(APIView):
    """Medication dispense list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: MedicationDispenseSerializer})
    def get(self, request):
        """List all medication dispenses."""
        queryset = MedicationDispense.objects.all()
        serializer = MedicationDispenseSerializer(queryset, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=MedicationDispenseSerializer,
        responses={201: MedicationDispenseSerializer},
    )
    def post(self, request):
        """Create a medication dispense."""
        serializer = MedicationDispenseSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class MedicationDispenseDetailView(APIView):
    """Medication dispense detail view."""

    def get_object(self, pk):
        """Get object method."""
        try:
            return MedicationDispense.objects.get(pk=pk)
        except MedicationDispense.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: MedicationDispenseSerializer})
    def get(self, request, pk=None):
        """Retrieve a medication dispense."""
        medication_dispense = self.get_object(pk)
        serializer = MedicationDispenseSerializer(medication_dispense)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request={200, MedicationDispenseSerializer})
    def patch(self, request, pk):
        """Update a medication dispense."""
        medication_dispense = self.get_object(pk)
        serializer = MedicationDispenseSerializer(
            medication_dispense, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete(self, request, pk=None):
        """Delete a medication dispense."""
        medication_dispense = self.get_object(pk)
        medication_dispense.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""immunization model choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class ImmunizationStatusChoices(models.TextChoices):
    """immunization status choices."""

    COMPLETED = "completed", _("Completed")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")
    NOT_DONE = "not-done", _("Not Done")



"""Slots models."""

from django.contrib.postgres.fields import ArrayField
from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    TimeStampedModel,
)
from dfhir.healthcareservices.models import HealthCareServiceCodeableReference
from dfhir.practitioners.models import Practitioner
from dfhir.schedules.models import Schedule

from .choices import AppointmentType, SlotStatus


class SlotReference(BaseReference):
    """Slot reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="slot_reference_identifier",
        null=True,
    )
    slot = models.ForeignKey(
        "Slot",
        on_delete=models.CASCADE,
        related_name="slot_reference_slot",
        null=True,
    )


class Slot(TimeStampedModel):
    """Slot model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="slot_identifier", blank=True
    )
    practitioner = models.ForeignKey(
        Practitioner, on_delete=models.DO_NOTHING, related_name="slot_practitioner"
    )
    service_category = models.ManyToManyField(
        CodeableConcept, related_name="slot_service_category", blank=True
    )
    service_type = models.ManyToManyField(
        HealthCareServiceCodeableReference, related_name="slot_service_type", blank=True
    )
    specialty = models.ManyToManyField(
        CodeableConcept, related_name="slot_specialty", blank=True
    )
    appointment_type = ArrayField(
        models.CharField(max_length=255, choices=AppointmentType.choices), null=True
    )
    schedule = models.ForeignKey(
        Schedule, on_delete=models.DO_NOTHING, null=True, related_name="slot_schedule"
    )
    start = models.DateTimeField(null=True)
    end = models.DateTimeField(null=True)
    overbooked = models.BooleanField(default=False)
    comment = models.TextField(null=True)
    status = models.CharField(
        max_length=255, choices=SlotStatus.choices, default=SlotStatus.FREE
    )
